//
// (c) 2006 DS Data Systems UK Ltd, All rights reserved.
//
// DS Data Systems and KonaKart and their respective logos, are 
// trademarks of DS Data Systems UK Ltd. All rights reserved.
//
// The information in this document is free software; you can redistribute 
// it and/or modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//

package com.konakart.bl.modules.payment.bluepay;

import java.math.BigDecimal;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.ResourceBundle;

import com.konakart.app.KKConfiguration;
import com.konakart.app.KKException;
import com.konakart.app.NameValue;
import com.konakart.app.Order;
import com.konakart.app.OrderTotal;
import com.konakart.app.PaymentDetails;
import com.konakart.appif.KKEngIf;
import com.konakart.bl.modules.BaseModule;
import com.konakart.bl.modules.ordertotal.OrderTotalMgr;
import com.konakart.bl.modules.payment.BasePaymentModule;
import com.konakart.bl.modules.payment.PaymentInfo;
import com.konakart.bl.modules.payment.PaymentInterface;

/**
 * BluePay payment module. This payment module allows for credit card credentials to be collected
 * directly from a KonaKart page. All communication to the BluePay server is done from the KonaKart
 * server.
 * 
 */
public class Bluepay extends BasePaymentModule implements PaymentInterface
{

    // Module name must be the same as the class name although it can be all in lowercase in order
    // to remain compatible with osCommerce.
    private static String code = "bluepay";

    private static int sortOrder = -1;

    // The BluePay Url used to POST the payment request.
    // "https://secure.bluepay.com/interfaces/bp20post"
    private static String bluepayRequestUrl;

    // The merchant key
    private static String bluepayAcctId;

    // Put the gateway in test mode
    private static boolean testMode = true;

    // Show the post code field on the UI
    private static boolean showPostcode = false;

    // Show the address field on the UI
    private static boolean showAddress = false;

    // Show the CVV field on the UI
    private static boolean showCVV = true;

    private static int zone;

    private static int orderStatusId;

    private static String bundleName = BaseModule.basePackage + ".payment.bluepay.Bluepay";

    private static HashMap<Locale, ResourceBundle> resourceBundleMap = new HashMap<Locale, ResourceBundle>();

    private static String mutex = "bluepayMutex";

    // Configuration Keys

    /**
     * Used to put the gateway online / offline
     */
    private final static String MODULE_PAYMENT_BLUEPAY_STATUS = "MODULE_PAYMENT_BLUEPAY_STATUS";

    /**
     * The BluePay zone, if greater than zero, should reference a GeoZone. If the DeliveryAddress of
     * the order isn't within that GeoZone, then we throw an exception
     */
    private final static String MODULE_PAYMENT_BLUEPAY_ZONE = "MODULE_PAYMENT_BLUEPAY_ZONE";

    private final static String MODULE_PAYMENT_BLUEPAY_ORDER_STATUS_ID = "MODULE_PAYMENT_BLUEPAY_ORDER_STATUS_ID";

    /**
     * The order for displaying this payment gateway on the UI
     */
    private final static String MODULE_PAYMENT_BLUEPAY_SORT_ORDER = "MODULE_PAYMENT_BLUEPAY_SORT_ORDER";

    /**
     * To perform a transaction in test mode
     */
    private final static String MODULE_PAYMENT_BLUEPAY_TESTMODE = "MODULE_PAYMENT_BLUEPAY_TESTMODE";

    /**
     * To show CVV field
     */
    private final static String MODULE_PAYMENT_BLUEPAY_SHOW_CVV = "MODULE_PAYMENT_BLUEPAY_SHOW_CVV";

    /**
     * The merchant key
     */
    private final static String MODULE_PAYMENT_BLUEPAY_ACCOUNT_ID = "MODULE_PAYMENT_BLUEPAY_ACCOUNT_ID";

    /**
     * The BluePay Url used to POST the payment request.
     * "https://secure.bluepay.com/interfaces/bp20post"
     */
    private final static String MODULE_PAYMENT_BLUEPAY_REQUEST_URL = "MODULE_PAYMENT_BLUEPAY_REQUEST_URL";

    // Message Catalogue Keys
    private final static String MODULE_PAYMENT_BLUEPAY_TEXT_TITLE = "module.payment.bluepay.text.title";

    private final static String MODULE_PAYMENT_BLUEPAY_TEXT_DESCRIPTION = "module.payment.bluepay.text.description";

    /**
     * Constructor
     * 
     * @param eng
     * 
     * @throws KKException
     */
    public Bluepay(KKEngIf eng) throws KKException
    {
        super.init(eng);

        // Initialise the static variables
        if (sortOrder == -1)
        {
            synchronized (mutex)
            {
                if (sortOrder == -1)
                {
                    setStaticVariables();
                }
            }
        }
    }

    /**
     * Sets some static variables during setup
     * 
     * @throws KKException
     * 
     */
    public void setStaticVariables() throws KKException
    {
        KKConfiguration conf;

        conf = getEng().getConfiguration(MODULE_PAYMENT_BLUEPAY_ACCOUNT_ID);
        if (conf == null)
        {
            throw new KKException("The Configuration MODULE_PAYMENT_BLUEPAY_ACCOUNT_ID must be set"
                    + " to the 12-digit BluePay Account ID");
        }
        bluepayAcctId = conf.getValue();

        conf = getEng().getConfiguration(MODULE_PAYMENT_BLUEPAY_REQUEST_URL);
        if (conf == null)
        {
            throw new KKException(
                    "The Configuration MODULE_PAYMENT_BLUEPAY_REQUEST_URL must be set to the URL for"
                            + " sending the request to BluePay. (i.e. https://secure.bluepay.com/interfaces/bp20post)");
        }
        bluepayRequestUrl = conf.getValue();

        conf = getEng().getConfiguration(MODULE_PAYMENT_BLUEPAY_ZONE);
        if (conf == null)
        {
            zone = 0;
        } else
        {
            zone = new Integer(conf.getValue()).intValue();
        }

        conf = getEng().getConfiguration(MODULE_PAYMENT_BLUEPAY_SORT_ORDER);
        if (conf == null)
        {
            sortOrder = 0;
        } else
        {
            sortOrder = new Integer(conf.getValue()).intValue();
        }

        conf = getEng().getConfiguration(MODULE_PAYMENT_BLUEPAY_ORDER_STATUS_ID);
        if (conf == null)
        {
            orderStatusId = 0;
        } else
        {
            orderStatusId = new Integer(conf.getValue()).intValue();
        }

        conf = getEng().getConfiguration(MODULE_PAYMENT_BLUEPAY_TESTMODE);
        if (conf == null)
        {
            testMode = true;
        } else
        {
            if (conf.getValue().trim().equalsIgnoreCase("false"))
            {
                testMode = false;
            } else
            {
                testMode = true;
            }
        }

        conf = getEng().getConfiguration(MODULE_PAYMENT_BLUEPAY_SHOW_CVV);
        if (conf == null)
        {
            showCVV = true;
        } else
        {
            if (conf.getValue().trim().equalsIgnoreCase("false"))
            {
                showCVV = false;
            } else
            {
                showCVV = true;
            }
        }

    }

    /**
     * Return a payment details object for the BluePay payment module
     * 
     * @param order
     * @param info
     * @return Returns information in a PaymentDetails object
     * @throws Exception
     */
    public PaymentDetails getPaymentDetails(Order order, PaymentInfo info) throws Exception
    {
        /*
         * The BluePay zone, if greater than zero, should reference a GeoZone. If the
         * DeliveryAddress of the order isn't within that GeoZone, then we throw an exception
         */
        if (zone > 0)
        {
            checkZone(info, zone);
        }

        // Get the scale for currency calculations
        int scale = new Integer(order.getCurrency().getDecimalPlaces()).intValue();

        // Get the resource bundle
        ResourceBundle rb = getResourceBundle(mutex, bundleName, resourceBundleMap, info
                .getLocale());
        if (rb == null)
        {
            throw new KKException("A resource file cannot be found for the country "
                    + info.getLocale().getCountry());
        }

        PaymentDetails pDetails = new PaymentDetails();
        pDetails.setCode(code);
        pDetails.setSortOrder(sortOrder);
        pDetails.setOrderStatusId(orderStatusId);
        pDetails.setPaymentType(PaymentDetails.SERVER_PAYMENT_GATEWAY);
        pDetails.setDescription(rb.getString(MODULE_PAYMENT_BLUEPAY_TEXT_DESCRIPTION));
        pDetails.setTitle(rb.getString(MODULE_PAYMENT_BLUEPAY_TEXT_TITLE));

        // Return now if the full payment details aren't required. This happens when the manager
        // just wants a list of payment gateways to display in the UI.
        if (info.isReturnDetails() == false)
        {
            return pDetails;
        }

        pDetails.setPostOrGet("post");
        pDetails.setRequestUrl(bluepayRequestUrl);

        List<NameValue> parmList = new ArrayList<NameValue>();

        // BluePay only requires details of the final price. No tax, subtotal etc.
        BigDecimal total = null;
        for (int i = 0; i < order.getOrderTotals().length; i++)
        {
            OrderTotal ot = (OrderTotal) order.getOrderTotals()[i];
            if (ot.getClassName().equals(OrderTotalMgr.ot_total))
            {
                total = ot.getValue().setScale(scale, BigDecimal.ROUND_HALF_UP);
            }
        }
        
        if (total == null)
        {
            throw new KKException("An Order Total was not found");
        }

        String mode = (testMode ? URLEncoder.encode("TEST", "UTF-8") : URLEncoder.encode("LIVE",
                "UTF-8"));
        parmList.add(new NameValue("ACCOUNT_ID", URLEncoder.encode(bluepayAcctId, "UTF-8")));
        parmList.add(new NameValue("TRANS_TYPE", URLEncoder.encode("SALE", "UTF-8")));
        parmList.add(new NameValue("AMOUNT", URLEncoder.encode(total.toString(), "UTF-8")));
        parmList.add(new NameValue("MODE", mode));
        parmList.add(new NameValue("CUSTOM_ID", URLEncoder.encode(Integer.toString(order.getId()),
                "UTF-8")));
        parmList.add(new NameValue("CUSTOM_ID2", URLEncoder.encode(Integer.toString(order
                .getCustomerId()), "UTF-8")));

        // Set the billing address
        parmList.add(new NameValue("COMPANY_NAME", URLEncoder.encode(order.getCustomerCompany(),
                "UTF-8")));
        parmList.add(new NameValue("ADDR1", URLEncoder.encode(order.getBillingStreetAddress(),
                "UTF-8")));
        parmList.add(new NameValue("CITY", URLEncoder.encode(order.getBillingCity(), "UTF-8")));
        parmList.add(new NameValue("STATE", URLEncoder.encode(order.getBillingState(), "UTF-8")));
        parmList.add(new NameValue("ZIP", URLEncoder.encode(order.getBillingPostcode(), "UTF-8")));
        parmList.add(new NameValue("PHONE", URLEncoder
                .encode(order.getCustomerTelephone(), "UTF-8")));
        parmList.add(new NameValue("EMAIL", URLEncoder.encode(order.getCustomerEmail(), "UTF-8")));
        parmList
                .add(new NameValue("COUNTRY", URLEncoder.encode(order.getBillingCountry(), "UTF-8")));

        // Put the parameters into an array
        NameValue[] nvArray = new NameValue[parmList.size()];
        parmList.toArray(nvArray);
        pDetails.setParameters(nvArray);

        // Set the fields that should be visible in the UI when gathering Credit Card details
        pDetails.setShowAddr(showAddress);
        pDetails.setShowCVV(showCVV);
        pDetails.setShowPostcode(showPostcode);
        pDetails.setShowType(false); // BluePay doesn't require the card type

        if (log.isDebugEnabled())
        {
            log.debug(pDetails.toString());
        }

        return pDetails;
    }

    /**
     * Returns true or false
     * 
     * @throws KKException
     */
    public boolean isAvailable() throws KKException
    {
        return isAvailable(getEng(), MODULE_PAYMENT_BLUEPAY_STATUS);
    }

}
